package it.openutils.web.jstl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.List;

import javax.servlet.jsp.PageContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;


/**
 * EL functions fo simple data access operations.
 * @author fgiust
 * @version $Revision: 149 $ ($Author: fgiust $)
 */
public class DaoFunctions
{

    /**
     * Logger.
     */
    private static Logger log = LoggerFactory.getLogger(DaoFunctions.class);

    /**
     * Call findAll on a given DAO and retuns the result. The findAll method is caled using reflection, so the DAO
     * doesn't need to implement any specific interface.
     * @param pageContext jsp page context
     * @param daoName dao name, without the "DAO" postfix, automatically added (e.g. "user" for "userDAO"). The full
     * name of the dao should be the name of a bean available in spring application context.
     * @return List returned by DAO.findAll().
     */
    public static List findAll(PageContext pageContext, String daoName) throws Throwable
    {

        String fullDaoName = daoName + "DAO";
        if (log.isDebugEnabled())
        {
            log.debug("calling dao:findall for [" + daoName + "]");
        }

        WebApplicationContext context = WebApplicationContextUtils.getWebApplicationContext(pageContext
            .getServletContext());

        Object dao = context.getBean(fullDaoName);

        Method m = dao.getClass().getMethod("findAll", null);

        List results = null;

        try
        {
            results = (List) m.invoke(dao, null);
        }
        catch (InvocationTargetException ex)
        {
            throw ex.getTargetException();
        }

        if (log.isDebugEnabled())
        {
            log.debug("Returning " + results);
        }

        return results;

    }

}
