package it.openutils.web.tag.debug;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;


/**
 * @author fgiust
 * @version $Revision: 253 $ ($Author: fgiust $)
 */
public class BaseDebugTag extends TagSupport
{

    /**
     * Stable <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 222;

    /**
     * codice html HTML_TABLE_START_4COL
     */
    protected static final String HTML_TABLE_START_4COL = "<table><thead><tr>"
        + "<th>key</th><th>value</th><th>class</th><th>size</th>"
        + "</tr></thead><tbody>";

    /**
     * codice html HTML_TABLE_START_3COL
     */
    protected static final String HTML_TABLE_START_3COL = "<table><thead><tr>"
        + "<th>key</th><th>value</th><th>class</th>"
        + "</tr></thead><tbody>";

    /**
     * codice html HTML_TABLE_START_2COL
     */
    protected static final String HTML_TABLE_START_2COL = "<table><thead><tr>"
        + "<th>key</th><th>value</th>"
        + "</tr></thead><tbody>";

    /**
     * codice html HTML_TABLE_END
     */
    protected static final String HTML_TABLE_END = "</tbody></table>";

    /**
     * Map contenente i valori per il debug
     */
    private Map debugMap;

    /**
     * write ClassName In Table?
     */
    private boolean outputClassName;

    /**
     * Setta la map con i valori da mostrare nel debug
     * @param debug map con i valori da mostrare nel debug
     */
    protected void setDebugMap(Map debug)
    {
        debugMap = debug;
    }

    /**
     * Scrivi il nome della classe nel debug
     * @param writeClassNameInTable <code>true</code> per far si che il nome della classe appaia nel debug
     */
    public void setOutputClassName(boolean writeClassNameInTable)
    {
        outputClassName = writeClassNameInTable;
    }

    /**
     * doEndTag. Scrive come tabella html la Map settata col metodo setDebugMap()
     * @return int
     * @throws JspException in caso di eccezioni nella scrittura sull'out
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    @Override
    public int doEndTag() throws JspException
    {

        if (debugMap.size() > 0)
        {
            StringBuffer buffer = new StringBuffer(debugMap.size() * 200);

            if (outputClassName)
            {
                buffer.append(HTML_TABLE_START_3COL);
            }
            else
            {
                buffer.append(HTML_TABLE_START_2COL);
            }

            Set set = debugMap.entrySet();
            Iterator iterator = set.iterator();
            int totalSize = 0;

            while (iterator.hasNext())
            {
                Map.Entry entry = (Map.Entry) iterator.next();

                Object value = entry.getValue();
                String key = (String) entry.getKey();

                buffer.append("<tr>");
                buffer.append("<td>" + key + "</td>");
                buffer.append("<td>" + escapeXml(value) + "</td>");
                if (outputClassName)
                {
                    if (value != null)
                    {
                        buffer.append("<td>" + value.getClass().getName() + "</td>");
                    }
                    else
                    {
                        buffer.append("<td>null</td>");
                    }
                }

                buffer.append("</tr>");
            }
            if (totalSize > 0)
            {
                buffer.append("<tr><td colspan=\"3\"><strong>total size (solo portlet webbank)</strong></td><td>"
                    + totalSize
                    + " byte</td></tr>");
            }

            buffer.append(HTML_TABLE_END);
            writeToOut(buffer);
        }
        else
        {
            writeToOut("nessun valore");
        }

        return super.doEndTag();
    }

    /**
     * trasforma tutte le occorrenze di &lt; in &amp;lt;
     * @param string stringa di cui fare l'escape - il metodo accetta un Object per comodit� (utilizza il toString()
     * dell'Object)
     * @return String
     */
    private String escapeXml(Object string)
    {
        if (string == null)
        {
            return null;
        }

        String stringed;

        // supporto per array di oggetti
        if (string.getClass().isArray())
        {
            stringed = ArrayUtils.toString(string);
        }
        // supporto per iterators
        else if (string instanceof Iterator)
        {
            List list = new ArrayList();
            Iterator iterator = (Iterator) string;
            while (iterator.hasNext())
            {
                list.add(iterator.next());
            }

            stringed = ArrayUtils.toString(list.toArray());
        }
        else
        {
            stringed = string.toString();
        }

        return StringUtils.replace(stringed, "<", "&lt;");
    }

    /**
     * metodo di utility per scrivere una semplice stringa sul JspWriter
     * @param string stringa da scrivere
     * @throws JspException nel caso si verifichi una IOException nella scrittura
     */
    protected void writeToOut(Object string) throws JspException
    {
        if (string != null)
        {
            try
            {
                pageContext.getOut().write(ObjectUtils.toString(string));
            }
            catch (IOException e)
            {
                throw new JspException("errore nella scrittura dell'output. String=[" + string + "]");
            }
        }

    }

}