package it.openutils.web.tag.debug;

import java.util.Enumeration;
import java.util.HashMap;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * @author fgiust
 * @version $Revision: 257 $ ($Author: fgiust $)
 */
public class DebugAttributesTag extends BaseDebugTag
{

    /**
     * Stable <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 222;

    /**
     * riferimento al Logger
     */
    private static Log log = LogFactory.getLog(DebugAttributesTag.class);

    /**
     * Scope. Pu� assumere i valori:
     * <ul>
     * <li>PageContext.PAGE_SCOPE</li>
     * <li>PageContext.REQUEST_SCOPE</li>
     * <li>PageContext.SESSION_SCOPE</li>
     * <li>PageContext.APPLICATION_SCOPE</li>
     * </ul>
     * Corrispondenti alle stringhe settate tramite l'attributo "scope" del tag:
     * <ul>
     * <li>page</li>
     * <li>request</li>
     * <li>session</li>
     * <li>application</li>
     * </ul>
     */
    private int scope = PageContext.PAGE_SCOPE;

    /**
     * Setta lo scope in cui recuperare la collezione su cui iterare
     * @param scopeString Scope dell'oggetto collection
     * @throws JspException se il valore passato non � tra: "page", "request", "session", "application"
     */
    public void setScope(String scopeString) throws JspException
    {

        if (scopeString.equalsIgnoreCase("page"))
        {
            scope = PageContext.PAGE_SCOPE;
        }
        else if (scopeString.equalsIgnoreCase("request"))
        {
            scope = PageContext.REQUEST_SCOPE;
        }
        else if (scopeString.equalsIgnoreCase("session"))
        {
            scope = PageContext.SESSION_SCOPE;
        }
        else if (scopeString.equalsIgnoreCase("application"))
        {
            scope = PageContext.APPLICATION_SCOPE;
        }
        else
        {
            log.error("Attributo \"scope\" non valido. Valore fornito= ["
                + scopeString
                + "]; valori ammessi: page, request, session, application");
            throw new JspException("Attributo \"scope\" non valido. Valore fornito= ["
                + scopeString
                + "]; valori ammessi: page, request, session, application");
        }
    }

    /**
     * crea l'HashMap che evrr� stampata nel metodo doEndTag()
     * @return int
     * @throws JspException eccezione generica
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    @Override
    public int doStartTag() throws JspException
    {

        HashMap map = new HashMap();
        try
        {
            Enumeration enm = pageContext.getAttributeNamesInScope(scope);

            while (enm.hasMoreElements())
            {
                String key = enm.nextElement().toString();

                if (!(scope == PageContext.PAGE_SCOPE && key.startsWith("javax.servlet")))
                {
                    Object value = pageContext.getAttribute(key, scope);
                    map.put(key, value);
                }
            }
        }
        catch (IllegalStateException e)
        {
            // ignore, no attributes available
        }

        setOutputClassName(true);
        setDebugMap(map);

        return super.doStartTag();
    }

}